@@ -436,6 +436,11 @@ final class JellyBeanAsyncCodec implements AsyncCodec {
     return mOutputBuffers[index];
   }
 
+  @Override
+  public MediaFormat getInputFormat() {
+    return null;
+  }
+
   @Override
   public void flush() {
     assertCallbacks();
