@@ -13,12 +13,12 @@
 
 #include "nsISupports.h"
 #include "nsMenuParentX.h"
-#include "nsMenuGroupOwnerX.h"
 #include "nsChangeObserver.h"
 #include "nsTArray.h"
 #include "nsString.h"
 
 class nsMenuBarX;
+class nsMenuGroupOwnerX;
 class nsMenuX;
 class nsIWidget;
 class nsIContent;
@@ -27,8 +27,8 @@ namespace mozilla {
 namespace dom {
 class Document;
 class Element;
-}
-}
+}  // namespace dom
+}  // namespace mozilla
 
 // ApplicationMenuDelegate is used to receive Cocoa notifications.
 @interface ApplicationMenuDelegate : NSObject <NSMenuDelegate> {
