@@ -20,6 +20,7 @@
 #include "nsINamed.h"
 #include "nsITimer.h"
 #include "nsIContent.h"
+#include "mozilla/dom/RustTypes.h"
 
 class nsIFrame;
 class nsPresContext;
@@ -27,17 +28,19 @@ class nsPresContext;
 namespace mozilla {
 class ComputedStyle;
 enum class StyleAppearance : uint8_t;
-class EventStates;
 }  // namespace mozilla
 
 class nsNativeTheme : public nsITimerCallback, public nsINamed {
  protected:
-  virtual ~nsNativeTheme() {}
+  virtual ~nsNativeTheme() = default;
 
   NS_DECL_ISUPPORTS
   NS_DECL_NSITIMERCALLBACK
   NS_DECL_NSINAMED
 
+  nsNativeTheme();
+
+ public:
   enum ScrollbarButtonType {
     eScrollbarButton_UpTop = 0,
     eScrollbarButton_Down = 1 << 0,
@@ -49,12 +52,9 @@ class nsNativeTheme : public nsITimerCallback, public nsINamed {
     eTreeSortDirection_Natural,
     eTreeSortDirection_Ascending
   };
-
-  nsNativeTheme();
-
   // Returns the content state (hover, focus, etc), see EventStateManager.h
-  mozilla::EventStates GetContentState(nsIFrame* aFrame,
-                                       mozilla::StyleAppearance aAppearance);
+  static mozilla::dom::ElementState GetContentState(
+      nsIFrame* aFrame, mozilla::StyleAppearance aAppearance);
 
   // Returns whether the widget is already styled by content
   // Normally called from ThemeSupportsWidget to turn off native theming
@@ -62,14 +62,10 @@ class nsNativeTheme : public nsITimerCallback, public nsINamed {
   bool IsWidgetStyled(nsPresContext* aPresContext, nsIFrame* aFrame,
                       mozilla::StyleAppearance aAppearance);
 
-  // Accessors to widget-specific state information
-
-  bool IsDisabled(nsIFrame* aFrame, mozilla::EventStates aEventStates);
-
   // RTL chrome direction
   static bool IsFrameRTL(nsIFrame* aFrame);
 
-  bool IsHTMLContent(nsIFrame* aFrame);
+  static bool IsHTMLContent(nsIFrame* aFrame);
 
   // button:
   bool IsDefaultButton(nsIFrame* aFrame) {
@@ -78,23 +74,6 @@ class nsNativeTheme : public nsITimerCallback, public nsINamed {
 
   bool IsButtonTypeMenu(nsIFrame* aFrame);
 
-  // checkbox:
-  bool IsChecked(nsIFrame* aFrame) {
-    return GetCheckedOrSelected(aFrame, false);
-  }
-
-  // radiobutton:
-  bool IsSelected(nsIFrame* aFrame) {
-    return GetCheckedOrSelected(aFrame, true);
-  }
-
-  bool IsFocused(nsIFrame* aFrame) {
-    return CheckBooleanAttr(aFrame, nsGkAtoms::focused);
-  }
-
-  // scrollbar button:
-  int32_t GetScrollbarButtonType(nsIFrame* aFrame);
-
   // tab:
   bool IsSelectedTab(nsIFrame* aFrame) {
     return CheckBooleanAttr(aFrame, nsGkAtoms::visuallyselected);
@@ -147,8 +126,6 @@ class nsNativeTheme : public nsITimerCallback, public nsINamed {
   bool IsHorizontal(nsIFrame* aFrame);
 
   // progressbar:
-  bool IsIndeterminateProgress(nsIFrame* aFrame,
-                               mozilla::EventStates aEventStates);
   bool IsVerticalProgress(nsIFrame* aFrame);
 
   // meter:
@@ -162,9 +139,6 @@ class nsNativeTheme : public nsITimerCallback, public nsINamed {
   // menupopup:
   bool IsSubmenu(nsIFrame* aFrame, bool* aLeftOfParent);
 
-  // True if it's not a menubar item or menulist item
-  bool IsRegularMenuItem(nsIFrame* aFrame);
-
   static bool CheckBooleanAttr(nsIFrame* aFrame, nsAtom* aAtom);
   static int32_t CheckIntAttr(nsIFrame* aFrame, nsAtom* aAtom,
                               int32_t defaultValue);
@@ -173,9 +147,6 @@ class nsNativeTheme : public nsITimerCallback, public nsINamed {
   static double GetProgressValue(nsIFrame* aFrame);
   static double GetProgressMaxValue(nsIFrame* aFrame);
 
-  bool GetCheckedOrSelected(nsIFrame* aFrame, bool aCheckSelected);
-  bool GetIndeterminate(nsIFrame* aFrame);
-
   bool QueueAnimatedContentForRefresh(nsIContent* aContent,
                                       uint32_t aMinimumFrameRate);
 
@@ -184,11 +155,10 @@ class nsNativeTheme : public nsITimerCallback, public nsINamed {
 
   bool IsRangeHorizontal(nsIFrame* aFrame);
 
-  // scrollbar
-  bool IsDarkBackground(nsIFrame* aFrame);
-  // custom scrollbar
-  typedef nscolor (*AutoColorGetter)(mozilla::ComputedStyle*);
-  bool IsWidgetScrollbarPart(mozilla::StyleAppearance aAppearance);
+  static bool IsDarkBackgroundForScrollbar(nsIFrame*);
+  static bool IsDarkBackground(nsIFrame*);
+
+  static bool IsWidgetScrollbarPart(mozilla::StyleAppearance);
 
  private:
   uint32_t mAnimatedContentTimeout;
