@@ -40,15 +40,6 @@
 #define MOUSE_INPUT_SOURCE() WinUtils::GetMouseInputSource()
 #define MOUSE_POINTERID() WinUtils::GetMousePointerID()
 
-/**************************************************************
- *
- * SECTION: enums
- *
- **************************************************************/
-
-// nsWindow::sCanQuit
-typedef enum { TRI_UNKNOWN = -1, TRI_FALSE = 0, TRI_TRUE = 1 } TriStateBool;
-
 /**************************************************************
  *
  * SECTION: constants
